@@ -198,9 +198,9 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
     b               = NULL,
     Z               = NULL,
     tolerance       = if (is.null(tolerance)) sqrt(.Machine$double.eps)
-                      else as.numeric(tolerance),
+    else as.numeric(tolerance),
     iteration_limit = if (is.null(iteration_limit)) 50L
-                      else as.integer(iteration_limit),
+    else as.integer(iteration_limit),
     r               = 0L,
     zhat            = NULL,
     final           = isTRUE(final),
@@ -253,7 +253,7 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
 .solve.instance <- function(object, problem="", C=NULL, S=NULL, M=NULL, b=NULL,
                             m=NULL, p=NULL, i=1L, j=1L, zero_diagonal=FALSE,
                             r=1L, Z=NULL, rcond=FALSE, tolerance=NULL,
-                            iteration_limit=NULL, final=TRUE, alpha=NULL, ...) {
+                            iteration_limit=NULL, final=NULL, alpha=NULL, ...) {
   # (A), (b) Construct a conformable canonical form for the CLSP estimator
   if (!is.null(C) || !is.null(M) || (!is.null(m) && !is.null(p))) {
     object <- canonize.clsp(object, problem, C, S, M, NULL, b,
@@ -264,7 +264,7 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
     stop(sprintf(paste0("The matrix A and vector b must have the same number ",
                         "of rows: A has %d, b has %d"),
                  nrow(object$A), nrow(object$b)))
-
+  
   # (zhat) (Iterated if r > 1) first-step estimate
   if      (r < 1L)   stop("Number of refinement iterations r must be \u2265 1.")
   if      (!is.null(Z))               object$Z <- Z
@@ -279,14 +279,14 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
     stop(sprintf(paste0("Matrix Z must be symmetric, idempotent and match ",
                         "the number of columns in A: expected (%d,%d), ",
                         "got (%d,%d)"), ncol(object$A), ncol(object$A),
-                                        nrow(object$Z), ncol(object$Z)))
+                 nrow(object$Z), ncol(object$Z)))
   for (n_iter in seq_len(if (nrow(object$A) > object$C_idx[1]) r else 1L)) {
     # save NRMSE from the previous step, construct Q and Z
     if (n_iter > 1L) {
       res        <- object$b - object$A %*% object$zhat
       nrmse_prev <- .nrmse.r2(object, res=res)
       Q          <- diag(as.numeric(-sign(res[(object$C_idx[1]    +
-                                               1):nrow(object$A), ,
+                                                 1):nrow(object$A), ,
                                               drop=FALSE])))
       object     <- canonize.clsp(object, problem, C, S, M, Q, b, m, p, i, j,
                                   zero_diagonal)
@@ -299,12 +299,16 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
       }
     }
     # solve via the Bott–Duffin inverse
-    object$zhat  <- (MASS::ginv(object$Z %*% crossprod(object$A) %*% object$Z,
-                                tol=if (is.null(rcond))
-                                    sqrt(.Machine$double.eps)           else
-                                    if (isTRUE(rcond)) object$tolerance else
-                                    rcond) %*% object$Z %*% t(object$A)) %*%
-                    object$b
+    object$zhat <- with(svd(M <- object$Z %*% crossprod(object$A) %*% object$Z),
+                        v %*% diag(ifelse(d > ((     if (isFALSE(rcond))
+                          max(dim(M))      *
+                            .Machine$double.eps
+                          else if (isTRUE(rcond))
+                            object$tolerance
+                          else rcond) * max(d)), 1/d, 0),
+                          length(d)) %*% t(u)            %*%
+                          object$Z %*% t(object$A)
+    ) %*% object$b
     object$nrmse <- .nrmse.r2(object, res=object$b - object$A %*% object$zhat)
     # break on convergence
     object$r <- n_iter
@@ -315,7 +319,7 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
     object$zhat <- NA_real_
     stop("Pseudoinverse estimate zhat failed")
   }
-
+  
   # (z) Final solution (if available), or set object$z = object$zhat
   if (!is.null(final)) object$final <- isTRUE(final)
   if (!is.null(alpha)) object$alpha <- max(0, min(1, as.numeric(alpha)))
@@ -332,7 +336,7 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
       s_cvx <- "OSQP"
     } else  {                                          # Elastic Net
       f_obj <- (1 - object$alpha) * CVXR::norm1(d_cvx) +
-               object$alpha * CVXR::sum_squares(d_cvx)
+        object$alpha * CVXR::sum_squares(d_cvx)
       s_cvx <- "SCS"
     }
     c_cvx <- list(A_csc %*% z_cvx == as.numeric(object$b))
@@ -353,44 +357,44 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
   } else {
     object$z   <- object$zhat
   }
-
+  
   # (x), (y) Variable and slack components of z
   object$x <- matrix(object$z[1:object$C_idx[2], ,        drop=FALSE],
                      nrow=if (!is.null(m)) m else object$C_idx[2],
-                     ncol=if (!is.null(p)) p else 1)
+                     ncol=if (!is.null(p)) p else 1,      byrow=TRUE)
   object$y <- (if (ncol(object$A) > object$C_idx[2])
     object$z[(object$C_idx[2] + 1):length(object$z), ,    drop=FALSE]
     else matrix(numeric(0), ncol=1))
-
+  
   # (kappaC), (kappaB), (kappaA) Condition numbers
   object$kappaC <- kappa(object$A[1:object$C_idx[1], ,    drop=FALSE])
   object$kappaB <- kappa(object$A %*% MASS::ginv(object$A[1:object$C_idx[1], ,
                                                           drop=FALSE]))
   object$kappaA <- kappa(object$A)
-
+  
   # (r2_partial), (nrmse_partial) M-block-based statistics
   if (nrow(object$A) > object$C_idx[1]) {
     object$r2_partial    <- .nrmse.r2(object, r2=TRUE, partial=TRUE)
     object$nrmse_partial <- .nrmse.r2(object,          partial=TRUE)
   }
-
+  
   # (z_lower), (z_upper) Condition-weighted confidence band
   b_norm <- sqrt(sum(object$b^2))
   dz     <- if (isTRUE(all.equal(b_norm, 0))) Inf                       else
-                                              object$kappaA                 *
-                                              sqrt(sum((object$b - object$A %*%
-                                              matrix(object$z, ncol=1))^2)) /
-                                              b_norm
+    object$kappaA                 *
+    sqrt(sum((object$b - object$A %*%
+                matrix(object$z, ncol=1))^2)) /
+    b_norm
   object$z_lower <- object$z * (1 - dz)
   object$z_upper <- object$z * (1 + dz)
-
+  
   # (x_lower), (x_upper), (y_lower), (y_upper)
   object$x_lower <- matrix(object$z_lower[1:object$C_idx[2], , drop=FALSE],
                            nrow=if (!is.null(m)) m else object$C_idx[2],
-                           ncol=if (!is.null(p)) p else 1)
+                           ncol=if (!is.null(p)) p else 1,     byrow=TRUE)
   object$x_upper <- matrix(object$z_upper[1:object$C_idx[2], , drop=FALSE],
                            nrow=if (!is.null(m)) m else object$C_idx[2],
-                           ncol=if (!is.null(p)) p else 1)
+                           ncol=if (!is.null(p)) p else 1,     byrow=TRUE)
   object$y_lower <- (if (ncol(object$A) > object$C_idx[2])
     object$z_lower[(object$C_idx[2] + 1):length(object$z), ,
                    drop=FALSE]
@@ -399,7 +403,7 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
     object$z_upper[(object$C_idx[2] + 1):length(object$z), ,
                    drop=FALSE]
     else matrix(numeric(0), ncol=1))
-
+  
   object
 }
 .solve <- function(object, tolerance=NULL, alpha=NULL, ...) {
@@ -409,7 +413,7 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
   dots$alpha     <- NULL
   to.alpha       <- function(a) max(0, min(1, as.numeric(a)))
   to.nrmse       <- function(n) if (is.finite(n)) n else Inf
-
+  
   # process alpha
   if      (!is.null(tolerance))
     object$tolerance <- tolerance
@@ -423,26 +427,26 @@ clsp <- function(problem="", C=NULL, S=NULL, M=NULL, b=NULL, m=NULL, p=NULL,
     idx    <- 1L
     for (a in alpha) {
       result[idx] <- suppressWarnings(to.nrmse(do.call(.solve.instance,
-                                      c(list(object, tolerance=object$tolerance,
-                                      final=TRUE, alpha=a), dots))$nrmse))
+                                     c(list(object, tolerance=object$tolerance,
+                                            final=NULL, alpha=a), dots))$nrmse))
       idx         <- idx + 1L
     }
     object$alpha  <- if (length(result) > 0L) alpha[which.min(result)]  else
-                                              NULL
+      NULL
   }
   if (is.null(object$alpha)) {                         # error rule
     nrmse.alpha0  <- suppressWarnings(to.nrmse(do.call(.solve.instance,
-                                      c(list(object, tolerance=object$tolerance,
-                                      final=TRUE, alpha=0), dots))$nrmse))
+                                     c(list(object, tolerance=object$tolerance,
+                                            final=NULL, alpha=0), dots))$nrmse))
     nrmse.alpha1  <- suppressWarnings(to.nrmse(do.call(.solve.instance,
-                                      c(list(object, tolerance=object$tolerance,
-                                      final=TRUE, alpha=1), dots))$nrmse))
+                                     c(list(object, tolerance=object$tolerance,
+                                            final=NULL, alpha=1), dots))$nrmse))
     denominator   <- nrmse.alpha0 + nrmse.alpha1 + object$tolerance
     object$alpha  <- if (is.finite(denominator) && denominator > 0)
                      to.alpha(nrmse.alpha0 / denominator)               else
-                     0.5
+        0.5
   }
-
+  
   do.call(.solve.instance, c(list(object, tolerance=object$tolerance,
-                                  final=TRUE, alpha=object$alpha), dots))
+                                  final=NULL, alpha=object$alpha), dots))
 }
